/*
 *
 *
 *    regex.cc
 *
 *    Source code for regular expression class.
 *
 *    =========================================================================
 *
 *    Copyright (C) 2001-2003 by Matt Roberts, KK5JY.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    =========================================================================
 *
 *
 */


#include <string>
#include <regex.h>

#include <my++/regex.h>

my::regex::regex (const std::string &ex) : cflags(REG_NOSUB) {
	if (regcomp (&re_exp, ex.c_str(), cflags) == 0)
		re_str = ex;
}

my::regex::regex (const my::regex &r) : cflags(REG_NOSUB) {
	copy(r);
}

my::regex::regex (void) : cflags(REG_NOSUB) {
	/* noop */
}

my::regex::~regex (void) {
	if (!re_str.empty())
		regfree(&re_exp);
}

bool my::regex::operator == (const my::regex &r) const {
	return ( (re_str == r.re_str) &&
			(cflags == r.cflags) );
}

bool my::regex::operator == (const std::string &s) const {
	return (regexec (&re_exp, s.c_str(), 0, 0, 0) == 0);
}

bool my::regex::operator != (const my::regex &r) const {
	return ( (re_str != r.re_str) ||
			(cflags != r.cflags) );
}

bool my::regex::operator != (const std::string &s) const {
	return (regexec (&re_exp, s.c_str(), 0, 0, 0) != 0);
}

const my::regex & my::regex::copy (const my::regex &r) {
	if (!re_str.empty()) {
		regfree(&re_exp);
	}

	if (!r.re_str.empty()) {
		if (regcomp (&re_exp, r.re_str.c_str(), r.cflags) == 0) {
			re_str = r.re_str;
			cflags = r.cflags;
		}
	}

	return *this;
}

/*
 *
 * recommended test application
 *
 */

/*
int main (int argc, char **argv) {
	if (argc < 3) {
		cerr << "Usage: regex [pattern] [string]" << endl;
		return 1;
	}

	regex r(argv[1]);

	if (r == argv[2])
		cerr << "Match." << endl;
	else
		cerr << "No match." << endl;

	return 0;
}
*/

